// -*- C++ -*-
/*!
 * @file  frm_ctrl.h
 * @brief motion control in task coordinate
 * @date  $Date$
 *
 * $Id$
 */

#ifndef FRM_CTRL_H
#define FRM_CTRL_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "com_frm_ctrlSVC_impl.h"

// </rtc-template>

class ExecRef;
class ExecCur;

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

using namespace RTC;

class frm_ctrl
  : public RTC::DataFlowComponentBase
{
  friend class ExecRef;
  friend class ExecCur;
 public:
  frm_ctrl(RTC::Manager* manager);
  ~frm_ctrl();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry() 
 virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  // virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedDoubleSeq m_ref_frm;
  InPort<TimedDoubleSeq> m_ref_frmIn;
  TimedDoubleSeq m_cur_frm;
  InPort<TimedDoubleSeq> m_cur_frmIn;
  
  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedDoubleSeq m_vel;
  OutPort<TimedDoubleSeq, NullBuffer> m_velOut;
  TimedLong m_state;
  OutPort<TimedLong, NullBuffer> m_stateOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  RTC::CorbaPort m_ComFrmCtrlPort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  ComFrmCtrlSVC_impl m_com_frm_ctrl;
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  // user

  ExecRef* m_eref;
  ExecCur* m_ecur;
  int act_flag, m_ref_flag, m_cur_flag;
  int first_flag;

  FRAME ref_frm, cur_frm;
  int state_old;

  int dummy;

};

class ExecRef : public RTC::OnWrite<RTC::TimedDoubleSeq> 
{
  frm_ctrl* m_ctrl;
public :
  ExecRef(frm_ctrl* comp) : m_ctrl(comp) {};
  void operator() (const RTC::TimedDoubleSeq& value) {
    int i,j;
    VECTOR p_vel, r_vel;
    //    std::cout << "get ref" << std::endl;
    if(m_ctrl->act_flag == 1) {
      m_ctrl->m_ref_flag = 1;
      m_ctrl->state_old = 0;
      m_ctrl->m_com_frm_ctrl.state = 0;
      m_ctrl->m_state.data = m_ctrl->state_old;
      m_ctrl->m_stateOut.write();
      for(i=0;i<3;i++) {
	for(j=0;j<3;j++) {
	  m_ctrl->ref_frm.mtr[i][j]=value.data[3*i+j];
	}
	m_ctrl->ref_frm.vtr[i]=value.data[9+i];
      }
      //      if(m_ctrl->m_cur_flag == 1) {
      //m_ctrl->m_com_frm_ctrl.calc_vel(m_ctrl->ref_frm, m_ctrl->cur_frm, p_vel, r_vel);
      //for(i=0;i<3;i++) {
      //  m_ctrl->m_vel.data[i]=p_vel[i];
      //  m_ctrl->m_vel.data[i+3]=r_vel[i];
      //}
      //m_ctrl->m_velOut.write();
      //if(m_ctrl->state_old != m_ctrl->m_com_frm_ctrl.state) {
      //  m_ctrl->state_old = m_ctrl->m_com_frm_ctrl.state;
      //  m_ctrl->m_state.data = m_ctrl->state_old;
      //  m_ctrl->m_stateOut.write();
      //}
      //}
    }
  }; // operator()
};


class ExecCur : public RTC::OnWrite<RTC::TimedDoubleSeq> 
{
  frm_ctrl* m_ctrl;
public :
  ExecCur(frm_ctrl* comp) : m_ctrl(comp) {};
  void operator() (const RTC::TimedDoubleSeq& value) {
    int i,j;
    VECTOR p_vel, r_vel;
    if(m_ctrl->act_flag == 1) {
      m_ctrl->m_cur_flag = 1;
      for(i=0;i<3;i++) {
	for(j=0;j<3;j++) {
	  m_ctrl->cur_frm.mtr[i][j]=value.data[3*i+j];
	}
	m_ctrl->cur_frm.vtr[i]=value.data[9+i];
      }
      if(m_ctrl->first_flag == 1) {
	m_ctrl->first_flag = 0;
        m_ctrl->m_ref_flag = 1;
	for(i=0;i<3;i++) {
	  for(j=0;j<3;j++) {
	    m_ctrl->ref_frm.mtr[i][j]=m_ctrl->cur_frm.mtr[i][j];
	  }
	  m_ctrl->ref_frm.vtr[i]=m_ctrl->cur_frm.vtr[i];
	}
      }
      if(m_ctrl->m_ref_flag == 1) {
	m_ctrl->m_com_frm_ctrl.calc_vel(m_ctrl->ref_frm, m_ctrl->cur_frm, p_vel, r_vel);
	for(i=0;i<3;i++) {
	  m_ctrl->m_vel.data[i]=p_vel[i];
	  m_ctrl->m_vel.data[i+3]=r_vel[i];
	}
	m_ctrl->m_velOut.write();
	if(m_ctrl->state_old != m_ctrl->m_com_frm_ctrl.state) {
	  m_ctrl->state_old = m_ctrl->m_com_frm_ctrl.state;
	  m_ctrl->m_state.data = m_ctrl->state_old;
	  m_ctrl->m_stateOut.write();
	}
      }
    }
  }; // operator()
};


extern "C"
{
  void frm_ctrlInit(RTC::Manager* manager);
};

#endif // FRM_CTRL_H
