// -*- C++ -*-
/*!
 * @file  mixer.cpp
 * @brief mix frame-vel and jacobian
 * $Date$
 *
 * $Id$
 */

#include "mixer.h"

// Module specification
// <rtc-template block="module_spec">
static const char* mixer_spec[] =
  {
    "implementation_id", "mixer",
    "type_name",         "mixer",
    "description",       "mix frame-vel and jacobian",
    "version",           "1.0",
    "vendor",            "Takashi Suehiro, AIST",
    "category",          "mixer",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables

    ""
  };
// </rtc-template>

mixer::mixer(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_velIn("vel", m_vel),
    m_jacobIn("jacob", m_jacob),
    m_j_listOut("j_list", m_j_list),
    
    // </rtc-template>
	dummy(0)
{
  m_evel=new ExecVel(this);
  m_velIn.setOnWrite(m_evel);
  m_ejacob=new ExecJacob(this);
  m_jacobIn.setOnWrite(m_ejacob);
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("vel", m_velIn);
  registerInPort("jacob", m_jacobIn);
  
  // Set OutPort buffer
  registerOutPort("j_list", m_j_listOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  m_j_list.data.length(48); // (1 + m) * n, where m = 7, n = 6


}

mixer::~mixer()
{
}


RTC::ReturnCode_t mixer::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  
  // </rtc-template>
  act_flag = 0;
  m_vel_flag = 0;
  m_jacob_flag = 0;
  //  m_vel.data.length(6);
  //  m_jacob.data.length(6*7);
  //  m_j_list.data.length(6*8);

  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t mixer::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t mixer::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t mixer::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t mixer::onActivated(RTC::UniqueId ec_id)
{
  act_flag = 1;
  return RTC::RTC_OK;
  /*
  int i;
  i = 0;
  std::cout << "checking inputs" << std::endl;
  for(i=0;i<1000;i++) {
    if((not m_velIn.isEmpty()) && (not m_jacobIn.isEmpty())) {
      return RTC::RTC_OK;
      std::cout << "ready" << std::endl;
    }
    usleep(1000);
  }
  std::cout << "lack of inputs" << std::endl;
  return RTC::RTC_ERROR;
  */
}

RTC::ReturnCode_t mixer::onDeactivated(RTC::UniqueId ec_id)
{
  act_flag = 0;
  m_vel_flag =0;
  m_jacob_flag =0;
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t mixer::onExecute(RTC::UniqueId ec_id)
{
  int i,j;
  int new_flag;

  new_flag = 0;
  //  std::cout << " go" << std::endl;
  if(not m_velIn.isEmpty()) {
    //    std::cout << " vel" << std::endl;
    new_flag = 1;
    m_velIn.read();
    for(i=0;i<6;i++) {
      m_j_list.data[i*8]=m_vel.data[i];
    }
  }
  if(not m_jacobIn.isEmpty()) {
    //    std::cout << " jacob" << std::endl;
    new_flag = 1;
    m_jacobIn.read();
    for(i=0;i<6;i++) {
      for(j=0;j<7;j++) {
	m_j_list.data[i*8+1+j]=m_jacob.data[i*7+j];
      }
    }
    //    std::cout << " j_list end" << std::endl;
  }
  if(new_flag == 1) {
    m_j_listOut.write();
    //    std::cout << " write j_list" << std::endl;
  }
  //  usleep(1000);
  usleep(10000);
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t mixer::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t mixer::onError(RTC::UniqueId ec_id)
{
  m_vel_flag =0;
  m_jacob_flag =0;
  act_flag = 0;
  return RTC::RTC_ERROR;
}


RTC::ReturnCode_t mixer::onReset(RTC::UniqueId ec_id)
{
  m_vel_flag =0;
  m_jacob_flag =0;
  act_flag = 0;
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t mixer::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t mixer::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void mixerInit(RTC::Manager* manager)
  {
    RTC::Properties profile(mixer_spec);
    manager->registerFactory(profile,
                             RTC::Create<mixer>,
                             RTC::Delete<mixer>);
  }
  
};


