// -*- C++ -*-
/*!
 * @file  pa10fk.h
 * @brief forward kinema of PA10
 * @date  $Date$
 *
 * $Id$
 */

#ifndef PA10FK_H
#define PA10FK_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "com_fkSVC_impl.h"
// fk class
// #include "fk.h" : inlcuded in com_fkSVC_impl.h


// </rtc-template>
//class pa10fk;
//class pa10fk : public RTC::DataFlowComponentBase;

class ExecFk;


// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

using namespace RTC;

class pa10fk
  : public RTC::DataFlowComponentBase
{
  friend class ExecFk;
 public:
  pa10fk(RTC::Manager* manager);
  ~pa10fk();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry()
 virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  // virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  // virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);

  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedDoubleSeq m_th;
  InPort<TimedDoubleSeq> m_thIn;
  
  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedDoubleSeq m_frame;
  OutPort<TimedDoubleSeq, NullBuffer> m_frameOut;
  TimedDoubleSeq m_jacob;
  OutPort<TimedDoubleSeq, NullBuffer> m_jacobOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  RTC::CorbaPort m_ComFkPort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  ComFkSVC_impl m_com_fk;
  
  // </rtc-template>
  
  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
  int act_flag;
  ExecFk *m_fk;
  //  ExecFk m_fk;
  int dummy;

};

class ExecFk : public RTC::OnWrite<RTC::TimedDoubleSeq>
{
  pa10fk* pfk;
public:
  ExecFk(pa10fk* comp) : pfk(comp) {};
  //  void set_comp(pa10fk *comp) 
  //  {
  //    pfk = comp;
  //  };
  void operator()(const RTC::TimedDoubleSeq& value)
  {
    int i,j;
    FRAME tmp_frm;
    Jacob tmp_jcb;
    REAL tmp_th[7];
    REAL tmp_abc[3];
    if(pfk->act_flag==1) {   // controlled by onActivated and onDeactivated
      //    m_thIn.read();
      for(i=0;i<7;i++) tmp_th[i]=value.data[i];
      pfk->m_com_fk.f_kinema.calc_fk(tmp_th);
      pfk->m_com_fk.f_kinema.link_frame(7, &tmp_frm);
      for(i=0;i<3;i++) {
	for(j=0;j<3;j++) {
	  pfk->m_frame.data[3*i+j]=tmp_frm.mtr[i][j];
	}
	pfk->m_frame.data[9+i]=tmp_frm.vtr[i];
      }
      pfk->m_com_fk.f_kinema.jacob(tmp_jcb);
      for(i=0;i<6;i++)
	for(j=0;j<7;j++)
	  pfk->m_jacob.data[7*i+j]=tmp_jcb[i][j];
      //      std::cout << "pre write" << std::endl;
      pfk->m_frameOut.write();
      pfk->m_jacobOut.write();
      mabc(tmp_frm.mtr, tmp_abc);
      /*
	std::cout << tmp_frm.vtr[0] << " ";
	std::couqt << tmp_frm.vtr[1] << " ";
	std::cout << tmp_frm.vtr[2] << " ";
	std::cout << tmp_abc[0]/PI*180 << " ";
	std::cout << tmp_abc[1]/PI*180 << " ";
	std::cout << tmp_abc[2]/PI*180 << " ";
	std::cout << std::endl;
      */
      //    std::cout << "write" << std::endl;
    }
  };
  //  usleep(1000);
  //  return RTC::RTC_OK;
};

extern "C"
{
  void pa10fkInit(RTC::Manager* manager);
};

#endif // PA10FK_H
