#!/usr/bin/env python
# -*- Python -*-

import sys
import time
import os

#
# set up user environment
#  RtmToolsDir, MyRtcDir, etc.
#
from set_env import *   

#
# import user tools
#
sys.path.append(".")
save_path = sys.path[:]
sys.path.append(RtmToolsDir+'/rtc_handle')
from rtc_handle import *
sys.path.append(MyRtcDir+'/tools/geo')
from geo import *
sys.path = save_path

#
# import stub files
# 
import _GlobalIDL

#
# run components
#
def run_components() :
  com_tr_jacob="cd "+MyRtcDir+"/tr_jacob; xterm -e ./tr_jacobComp"
  com_ct="cd "+MyRtcDir+"/coord_trans; xterm -e ./coord_transComp"
  com_jinv="cd "+MyRtcDir+"/jinv; xterm -e ./jinvComp"
  com_mixer="cd "+MyRtcDir+"/mixer_py; xterm -e python mixer.py"
  com_j_jacob="cd "+MyRtcDir+"/j_jacob; xterm -e ./j_jacobComp"
  com_slider="cd "+MyRtcDir+"/slider; xterm -e python sliderComp.py"
  com_frm_ctrl="cd "+MyRtcDir+"/frm_ctrl; xterm -e ./frm_ctrlComp"
  com_pa10fk="cd "+MyRtcDir+"/pa10fk; xterm -e ./pa10fkComp"
  com_vel_7dof="cd "+MyRtcDir+"/vel_sim; xterm -e ./vel_7dofComp"
  os.popen2(com_tr_jacob)
  os.popen2(com_ct)
  os.popen2(com_jinv)
  os.popen2(com_j_jacob)
  os.popen2(com_slider)
  os.popen2(com_frm_ctrl)
  os.popen2(com_mixer)
  os.popen2(com_pa10fk)
  os.popen2(com_vel_7dof)
  com_pa10disp="cd "+MyRtcDir+"/pa10disp_py; xterm -e python pa10disp.py"
  os.popen2(com_pa10disp)
  com_move2="cd "+MyRtcDir+"/move_py; xterm -e python move.py"
  os.popen2(com_move2)

def assign_variables(env) :
  # rtc
  global vel_7dof, pa10fk, frm_ctrl, coord_trans, tr_jacob
  global mixer, j_jacob, jinv, pa10disp, move, slider
  vel_7dof = env.name_space["localhost:9876"].rtc_handles['vel_7dof0.rtc']
  pa10fk = env.name_space["localhost:9876"].rtc_handles['pa10fk0.rtc']
  frm_ctrl = env.name_space["localhost:9876"].rtc_handles['frm_ctrl0.rtc']
  coord_trans = env.name_space["localhost:9876"].rtc_handles['coord_trans0.rtc']
  tr_jacob =  env.name_space["localhost:9876"].rtc_handles['tr_jacob0.rtc']
  j_jacob = env.name_space["localhost:9876"].rtc_handles['j_jacob0.rtc']
  slider = env.name_space["localhost:9876"].rtc_handles['sliderComp0.rtc']
  mixer = env.name_space["localhost:9876"].rtc_handles['mixer0_py.rtc']
  jinv =env.name_space["localhost:9876"].rtc_handles['jinv0.rtc']
  pa10disp = env.name_space["localhost:9876"].rtc_handles['pa10disp0.rtc']
  move = env.name_space["localhost:9876"].rtc_handles['move0.rtc']
  # narrow services
  pa10fk.services['ComFk'].provided['com_fk'].narrow_ref(globals())
  move.services['ComMv'].provided['com_move'].narrow_ref(globals())
  # data & class
  global c_frame, m_tool_frm, m_target_frm
  c_frame=_GlobalIDL.Frame
  m_tool_frm = c_frame(MATRIX(),VECTOR())
  m_target_frm = c_frame(MATRIX(),VECTOR())

def make_connectors(env):
  env.con_list=[]
#0
  con = IOConnector([vel_7dof.outports['th'],pa10fk.inports['th']])
  env.con_list.append(con)
#1
#  con = IOConnector([vel_7dof.outports['th2'],pa10disp.inports['th']])
#  env.con_list.append(con)
  con = IOConnector([vel_7dof.outports['th'],pa10disp.inports['th']])
  env.con_list.append(con)
#2
  con = IOConnector([pa10fk.outports['frame'],coord_trans.inports['in']])
  env.con_list.append(con)
#3
  con = IOConnector([coord_trans.outports['out'],frm_ctrl.inports['cur_frm']])
  env.con_list.append(con)
#4
  con = IOConnector([pa10fk.outports['jacob'],tr_jacob.inports['jacob']])
  env.con_list.append(con)
#5
  con = IOConnector([frm_ctrl.outports['vel'],tr_jacob.inports['vel']])
  env.con_list.append(con)
#6
  con = IOConnector([tr_jacob.outports['y_jacob'],mixer.inports['cartesian']])
  env.con_list.append(con)
#7
  con = IOConnector([j_jacob.outports['y_jacob'],mixer.inports['joint']])
  env.con_list.append(con)
#8
  con = IOConnector([slider.outports['out'],j_jacob.inports['vel']])
  env.con_list.append(con)
#9
  con = IOConnector([mixer.outports['out'],jinv.inports['y_jacob']])
  env.con_list.append(con)
#10
  con = IOConnector([move.outports['frame'],frm_ctrl.inports['ref_frm']])
  env.con_list.append(con)
#11
  con = IOConnector([frm_ctrl.outports['state'],move.inports['state']])
  env.con_list.append(con)
#12
  con = IOConnector([jinv.outports['vel'],vel_7dof.inports['vel']])
  env.con_list.append(con)
#13
  con = IOConnector([slider.outports['out'],vel_7dof.inports['vel']])
  env.con_list.append(con)

def connect_components(env) :
  for con in env.con_list :
    con.connect()

#
# connection control
#
def disconnect_all(env) :
  for con in env.con_list :
    con.disconnect()
def connect_list(env,ll) :
  for i in ll :
    env.con_list[i].connect()
def disconnect_list(env,ll) :
  for i in ll :
    env.con_list[i].disconnect()
#
# mode control
##
def mode_joint() :
  disconnect_all(env)
  connect_list(env,[1])
#
def mode_rmrc() :
  frm_ctrl.deactivate()
  disconnect_all(env)
  set_tool(0.0,0.0,165.0)
  connect_list(env,[0,1,2,3,4,5,6,7,8,9,10,11,12])
  frm_ctrl.activate()
#
def mode_slider() :
  slider.deactivate()
  disconnect_all(env)
  connect_list(env,[1,13])
  slider.activate()
#
def activate_components() :
  vel_7dof.activate()
  time.sleep(1)
  pa10disp.activate()
  pa10fk.activate()
  coord_trans.activate()
  move.activate()
  time.sleep(5)
  frm_ctrl.activate()
  time.sleep(5)
  slider.activate()
  j_jacob.activate()
  tr_jacob.activate()
  time.sleep(3)
  mixer.activate()
  time.sleep(3)
  jinv.activate()

def deactivate_components() :
  vel_7dof.deactivate()
  pa10disp.deactivate()
  pa10fk.deactivate()
  coord_trans.deactivate()
  move.deactivate()
  frm_ctrl.deactivate()
  slider.deactivate()
  j_jacob.deactivate()
  tr_jacob.deactivate()
  mixer.deactivate()
  jinv.deactivate()

#
# joint motion
#
def move_joint(vel_7dof, angles, k_pos=1.0, v_max=0.1, lim=0.001) :
  while 1:
    cur=vel_7dof.outports['th'].read().data
    if len(cur) != 7 :
      print cur
    else :
      diff=map(lambda x,y:x-y, cur, angles)
      dm=max(map(abs,diff))
      if dm < lim :
        vel = [0.0,0.0,0.0,0.0,0.0,0.0,0.0]
        vel_7dof.inports['vel'].write(vel)
        break
      vel=map(lambda x: - k_pos * x, diff)
      vm = max(map(abs,vel))
      if vm > v_max :
        vel=map(lambda x : x * v_max/vm, vel)
      vel_7dof.inports['vel'].write(vel)
    time.sleep(0.02)
  return cur
#
# some joint configurations
#
j_park=[0.0,0.0,0.0,0.0,0.0,0.0,0.0]
j_ready=[0.0,0.5,0.0,1.6,0.0,1.0,0.0]
#j1=[pi/4,0.5,0.0,pi/6,0.0,1.0,0.0]
#j2=[0.0,0.5,0.0,pi/6,0.0,0.0,0.0]
#
env = RtmEnv(sys.argv,["localhost:9876"])
run_components()
time.sleep(10)
raw_input("are you sure that every rtc needed are running?")
for ns in env.name_space :
  env.name_space[ns].list_obj()
assign_variables(env)
slider_conf="[('j1',-1,1,0.01,200),('j2',-1,1,0.01,200),('j3',-1,1,0.01,200),('j4',-1,1,0.01,200),('j5',-1,1,0.01,200),('j6',-1,1,0.01,200),('j7',-1,1,0.01,200)]"
slider.set_conf_activate('default','channels',slider_conf)
mixer_com=mixer.services['prov'].provided['com_mixer']
jw1=[1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0]
cw1=[1.0, 1.0, 1.0, 1.0, 1.0, 1.0]
#
global pa, pb, pc
#pa = FRAME(xyzabc=[600.0, 0.0, 220.0, 0.0, 0.0, pi/4])
#pb = FRAME(xyzabc=[600.0, 0.0, 500.0, 0.0, 0.0, pi/4])
#pc = FRAME(xyzabc=[600.0, 300.0, 300.0, 0.0, 0.0, pi/4])
pa = FRAME(xyzabc=[600.0, 0.0, 220.0, 0.0, pi, pi/4])
pb = FRAME(xyzabc=[600.0, 0.0, 500.0, 0.0, pi, pi/4])
pc = FRAME(xyzabc=[600.0, 300.0, 300.0, 0.0, pi, pi/4])
def demo(mv_rtc, pos_list) :
     for pos in pos_list :
       go_to(mv_rtc, pos)
def go_to(mv_rtc, pos) :
   m_target_frm.mat = pos.mat
   m_target_frm.pos = pos.vec
#   m_target_frm = c_frame(pos.mat,pos.vec)
   mv_rtc.services['ComMv'].provided['com_move'].ref.move_to(m_target_frm)
#
def cleanup_demo2() :
   env.con_list[10].connect()
   env.con_list[11].connect()
def setup_demo2() :
   env.con_list[10].disconnect()
   env.con_list[11].disconnect()
def demo2(ctrl_rtc, pos_list) :
     for pos in pos_list :
       go_to2(ctrl_rtc, pos)
def go_to2(ctrl_rtc, pos) :
      frm_data=range(12)
      for i in range(3) :
        for j in range(3) :
          frm_data[3*i+j]=pos.mat[i][j]
        frm_data[9+i]=pos.vec[i]
      ctrl_rtc.inports['ref_frm'].write(frm_data)
      time.sleep(0.5)
      state = 0
      print state
      while not state :
        tmp = ctrl_rtc.outports['state'].read().data
        if (tmp == 0 or tmp == 1) :
            if not state == tmp :
               state = tmp
               print state
        time.sleep(0.01)
#
def set_tool(x,y,z) :
   m_tool_frm.pos[0] = x
   m_tool_frm.pos[1] = y
   m_tool_frm.pos[2] = z
   pa10fk.services['ComFk'].provided['com_fk'].ref.set_tool(m_tool_frm)
#
make_connectors(env)
#connect_components(env)
activate_components()
