// -*- C++ -*-
/*!
 * @file  tr_jacob.cpp * @brief tr_jacob * $Date$ 
 *
 * $Id$ 
 */
#include "tr_jacob.h"

// Module specification
// <rtc-template block="module_spec">
static const char* tr_jacob_spec[] =
  {
    "implementation_id", "tr_jacob",
    "type_name",         "tr_jacob",
    "description",       "tr_jacob",
    "version",           "1.0",
    "vendor",            "Takashi Suehiro, AIST",
    "category",          "coord_trans",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    ""
  };
// </rtc-template>

tr_jacob::tr_jacob(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_velIn("vel", m_vel),
    m_jacobIn("jacob", m_jacob),
    m_y_jacobOut("y_jacob", m_y_jacob),
    m_ComCtPort("ComCt")

    // </rtc-template>
{
  m_evel=new ExecVel(this);
  m_velIn.setOnWrite(m_evel);
  m_ejacob=new ExecJacob(this);
  m_jacobIn.setOnWrite(m_ejacob);
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("vel", m_velIn);
  registerInPort("jacob", m_jacobIn);

  // Set OutPort buffer
  registerOutPort("y_jacob", m_y_jacobOut);

  // Set service provider to Ports
  m_ComCtPort.registerProvider("com_ct", "ComCt", m_com_ct);

  // Set service consumers to Ports

  // Set CORBA Service Ports
  registerPort(m_ComCtPort);

  // </rtc-template>

  m_y_jacob.data.length(48); // (1 + m) * n, where m = 7, n = 6
}

tr_jacob::~tr_jacob()
{
}


RTC::ReturnCode_t tr_jacob::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable

  // </rtc-template>
  act_flag = 0;
  m_vel_flag = 0;
  m_jacob_flag = 0;
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t tr_jacob::onFinalize()
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t tr_jacob::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t tr_jacob::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
RTC::ReturnCode_t tr_jacob::onActivated(RTC::UniqueId ec_id)
{
  act_flag = 1;
  return RTC::RTC_OK;
}
RTC::ReturnCode_t tr_jacob::onDeactivated(RTC::UniqueId ec_id)
{
  act_flag = 0;
  m_vel_flag =0;
  m_jacob_flag =0;
  return RTC::RTC_OK;
}
RTC::ReturnCode_t tr_jacob::onExecute(RTC::UniqueId ec_id)
{
  usleep(10000);
  return RTC::RTC_OK;
}
/*
RTC::ReturnCode_t tr_jacob::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
RTC::ReturnCode_t tr_jacob::onError(RTC::UniqueId ec_id)
{
  m_vel_flag =0;
  m_jacob_flag =0;
  act_flag = 0;
  return RTC::RTC_OK;
}
RTC::ReturnCode_t tr_jacob::onReset(RTC::UniqueId ec_id)
{
  m_vel_flag =0;
  m_jacob_flag =0;
  act_flag = 0;
  return RTC::RTC_OK;
}
/*
RTC::ReturnCode_t tr_jacob::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/
/*
RTC::ReturnCode_t tr_jacob::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


extern "C"
{
 
  void tr_jacobInit(RTC::Manager* manager)
  {
    RTC::Properties profile(tr_jacob_spec);
    manager->registerFactory(profile,
                             RTC::Create<tr_jacob>,
                             RTC::Delete<tr_jacob>);
  }
  
};



